<?php

namespace Ulmus\Api\Response;

use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\StreamInterface;
use Psr\Http\Message\UriInterface;
use Ulmus\Api\Common\HttpHeaderEnum;
use Ulmus\Api\Common\JsonMessageTrait;
use Ulmus\Api\Common\MethodEnum;
use Ulmus\Api\Stream\JsonStream;
use Ulmus\Api\Stream\Stream;
use Ulmus\Api\Common\StreamOutputInterface;

class JsonResponse extends Response implements StreamOutputInterface
{
    use JsonMessageTrait;

    public function __construct(null|StreamInterface $stream = null, int $statusCode = 200, array $headers = [])
    {
        parent::__construct($stream, $statusCode, $headers + [ 'Content-Type' => "application/json" ]);
    }

    public static function fromResponse(ResponseInterface $response) : static
    {
        return new static($response->getBody(), $response->getStatusCode(), $response->getHeaders());
    }

    public static function createFrom(mixed $content, int $encodingOptions = JsonStream::DEFAULT_JSON_ENCODING_FLAGS) : ResponseInterface
    {
        return new static(JsonStream::fromContent($content, $encodingOptions));
    }
}